#ifndef __MINIXML_BI__
#define __MINIXML_BI__

' Mini-XML, a small XML-like file parsing library.
' Copyright 2003-2014 by Michael R Sweet.
' 

#include once "crt/stdio.bi"
#include once "crt/stdlib.bi"
#include once "crt/string.bi"
#include once "crt/ctype.bi"
#include once "crt/errno.bi"

#ifndef __FB_64BIT__
# inclib "mxml-2.9-32"
#else
# inclib "mxml-2.9-64"
#endif

extern "C"


const MXML_MAJOR_VERSION   = 2
const MXML_MINOR_VERSION   = 9

const MXML_TAB             = 8

const MXML_NO_CALLBACK     = 0
const MXML_TEXT_CALLBACK   = 0
const MXML_NO_PARENT       = 0

const MXML_DESCEND         = 1
const MXML_NO_DESCEND      = 0
const MXML_DESCEND_FIRST   = -1

const MXML_WS_BEFORE_OPEN  = 0
const MXML_WS_AFTER_OPEN   = 1
const MXML_WS_BEFORE_CLOSE = 2
const MXML_WS_AFTER_CLOSE  = 3

const MXML_ADD_BEFORE      = 0
const MXML_ADD_AFTER       = 1
#define MXML_ADD_TO_PARENT NULL

type mxml_sax_event_t as long
const as mxml_sax_event_t MXML_SAX_CDATA         = 0
const as mxml_sax_event_t MXML_SAX_COMMENT       = 1
const as mxml_sax_event_t MXML_SAX_DATA          = 2
const as mxml_sax_event_t MXML_SAX_DIRECTIVE     = 3
const as mxml_sax_event_t MXML_SAX_ELEMENT_CLOSE = 4
const as mxml_sax_event_t MXML_SAX_ELEMENT_OPEN  = 5

type mxml_type_t as long
const as mxml_type_t MXML_IGNORE = -1
const as mxml_type_t MXML_ELEMENT = 0
const as mxml_type_t MXML_INTEGER = 1
const as mxml_type_t MXML_OPAQUE  = 2
const as mxml_type_t MXML_REAL    = 3
const as mxml_type_t MXML_TEXT    = 4
const as mxml_type_t MXML_CUSTOM  = 5

type mxml_attr_t
  as zstring ptr name
  as zstring ptr value
end type

type mxml_element_t
  as zstring ptr     name
  as long            num_attrs
  as mxml_attr_t ptr attrs
end type

type mxml_text_t
  as long        whitespace
  as zstring ptr string 
end type

type mxml_custom_destroy_cb_t as sub(byval as any ptr)
type mxml_custom_t
  as any ptr                  data 
  as mxml_custom_destroy_cb_t destroy
end type

union mxml_value_t
  as mxml_element_t element 
  as long           integer 
  as zstring ptr    opaque 
  as double         real 
  as mxml_text_t    text 
  as mxml_custom_t  custom
end union

type mxml_node_t
  as mxml_type_t     type
  as mxml_node_t ptr next 
  as mxml_node_t ptr prev 
  as mxml_node_t ptr parent 
  as mxml_node_t ptr child 
  as mxml_node_t ptr last_child 
  as mxml_value_t    value 
  as long            ref_count 
  as any ptr         user_data 
end type

type mxml_index_t
  as zstring ptr         attr 
  as long                num_nodes 
  as long                alloc_nodes 
  as long                cur_node 
  as mxml_node_t ptr ptr nodes
end type

type mxml_error_cb_t       as sub     (byval as const zstring ptr)

type mxml_custom_load_cb_t as function(byval as mxml_node_t ptr, byval as const zstring ptr) as long
type mxml_custom_save_cb_t as function(byval as mxml_node_t ptr) as zstring ptr

type mxml_entity_cb_t      as function(byval as const zstring ptr) as long
type mxml_load_cb_t        as function(byval as mxml_node_t ptr) as mxml_type_t
type mxml_save_cb_t        as function(byval as mxml_node_t ptr, byval as long) as const zstring ptr
type mxml_sax_cb_t         as sub     (byval as mxml_node_t ptr, byval as mxml_sax_event_t, byval as any ptr)

declare function mxmlLoadFd(byval top as mxml_node_t ptr, byval fd as long, cb as mxml_load_cb_t) as mxml_node_t ptr
declare function mxmlLoadFile(byval top as mxml_node_t ptr, byval fp as FILE ptr, cb as mxml_load_cb_t) as mxml_node_t ptr
declare function mxmlLoadString(byval top as mxml_node_t ptr, byval s as const zstring ptr, cb as mxml_load_cb_t) as mxml_node_t ptr

declare function mxmlNewXML(byval version as const zstring ptr) as mxml_node_t ptr
declare sub      mxmlDelete(byval node as mxml_node_t ptr)

declare sub      mxmlAdd(byval parent as mxml_node_t ptr, byval where as long, byval child as mxml_node_t ptr, byval node as mxml_node_t ptr)

declare sub      mxmlElementDeleteAttr(byval node as mxml_node_t ptr, byval name as const zstring ptr)
declare function mxmlElementGetAttr(byval node as mxml_node_t ptr, byval name as const zstring ptr) as const zstring ptr
declare sub      mxmlElementSetAttr(byval node as mxml_node_t ptr, byval name as const zstring ptr, byval value as const zstring ptr)
declare sub      mxmlElementSetAttrf(byval node as mxml_node_t ptr, byval name as const zstring ptr, byval format as const zstring ptr, ...)

declare function mxmlEntityAddCallback(byval cb as mxml_entity_cb_t) as long
declare sub      mxmlEntityRemoveCallback(byval cb as mxml_entity_cb_t)

declare function mxmlEntityGetName(byval val as long) as const zstring ptr
declare function mxmlEntityGetValue(byval name as const zstring ptr) as long


declare function mxmlFindElement(byval node as mxml_node_t ptr, byval top as mxml_node_t ptr, byval name as const zstring ptr, byval attr as const zstring ptr, byval value as const zstring ptr, byval descend as long) as mxml_node_t ptr
declare function mxmlFindPath(byval node as mxml_node_t ptr, byval path as const zstring ptr) as mxml_node_t ptr

declare function mxmlGetParent(byval node as mxml_node_t ptr) as mxml_node_t ptr
declare function mxmlGetPrevSibling(byval node as mxml_node_t ptr) as mxml_node_t ptr

declare function mxmlGetCDATA(byval node as mxml_node_t ptr) as const zstring ptr
declare function mxmlGetCustom(byval node as mxml_node_t ptr) as const any ptr

declare function mxmlGetFirstChild(byval node as mxml_node_t ptr) as mxml_node_t ptr
declare function mxmlGetInteger(byval node as mxml_node_t ptr) as long
declare function mxmlGetLastChild(byval node as mxml_node_t ptr) as mxml_node_t ptr
declare function mxmlGetNextSibling(byval node as mxml_node_t ptr) as mxml_node_t ptr

declare function mxmlGetOpaque(byval node as mxml_node_t ptr) as const zstring ptr

declare function mxmlGetReal(byval node as mxml_node_t ptr) as double
declare function mxmlGetRefCount(byval node as mxml_node_t ptr) as long
declare function mxmlGetText(byval node as mxml_node_t ptr, byval whitespace as long ptr) as const zstring ptr
declare function mxmlGetType(byval node as mxml_node_t ptr) as mxml_type_t
declare function mxmlGetUserData(byval node as mxml_node_t ptr) as any ptr

declare function mxmlIndexNew(byval node as mxml_node_t ptr, byval element as const zstring ptr, byval attr as const zstring ptr) as mxml_index_t ptr
declare sub      mxmlIndexDelete(byval ind as mxml_index_t ptr)

declare function mxmlIndexEnum(byval ind as mxml_index_t ptr) as mxml_node_t ptr
declare function mxmlIndexFind(byval ind as mxml_index_t ptr, byval element as const zstring ptr, byval value as const zstring ptr) as mxml_node_t ptr
declare function mxmlIndexGetCount(byval ind as mxml_index_t ptr) as long
declare function mxmlIndexReset(byval ind as mxml_index_t ptr) as mxml_node_t ptr

' element
declare function mxmlGetElement(byval node as mxml_node_t ptr) as const zstring ptr
declare function mxmlNewElement(byval parent as mxml_node_t ptr, byval name as const zstring ptr) as mxml_node_t ptr
declare function mxmlNewCDATA(byval parent as mxml_node_t ptr, byval string as const zstring ptr) as mxml_node_t ptr
declare function mxmlNewInteger(byval parent as mxml_node_t ptr, byval integer as long) as mxml_node_t ptr
declare function mxmlNewOpaque(byval parent as mxml_node_t ptr, byval opaque as const zstring ptr) as mxml_node_t ptr
declare function mxmlNewReal(byval parent as mxml_node_t ptr, byval real as double) as mxml_node_t ptr
declare function mxmlNewText(byval parent as mxml_node_t ptr, byval whitespace as long, byval string as const zstring ptr) as mxml_node_t ptr
declare function mxmlNewTextf(byval parent as mxml_node_t ptr, byval whitespace as long, byval format as const zstring ptr, ...) as mxml_node_t ptr
declare function mxmlNewCustom(byval parent as mxml_node_t ptr, byval data as any ptr, byval destroy as mxml_custom_destroy_cb_t) as mxml_node_t ptr

declare function mxmlRelease(byval node as mxml_node_t ptr) as long
declare sub      mxmlRemove(byval node as mxml_node_t ptr)
declare function mxmlRetain(byval node as mxml_node_t ptr) as long

declare function mxmlSaveAllocString(byval node as mxml_node_t ptr, byval cb as mxml_save_cb_t) as zstring ptr
declare function mxmlSaveFd(byval node as mxml_node_t ptr, byval fd as long, byval cb as mxml_save_cb_t) as long
declare function mxmlSaveFile(byval node as mxml_node_t ptr, byval fp as FILE ptr, byval cb as mxml_save_cb_t) as long
declare function mxmlSaveString(byval node as mxml_node_t ptr, byval buffer as zstring ptr, byval bufsize as long, byval cb as mxml_save_cb_t) as long

declare function mxmlSAXLoadFd(byval top as mxml_node_t ptr, byval fd as long, byval cb as function(byval as mxml_node_t ptr) as mxml_type_t, byval sax as mxml_sax_cb_t, byval sax_data as any ptr) as mxml_node_t ptr
declare function mxmlSAXLoadFile(byval top as mxml_node_t ptr, byval fp as FILE ptr, byval cb as function(byval as mxml_node_t ptr) as mxml_type_t, byval sax as mxml_sax_cb_t, byval sax_data as any ptr) as mxml_node_t ptr
declare function mxmlSAXLoadString(byval top as mxml_node_t ptr, byval s as const zstring ptr, byval cb as function(byval as mxml_node_t ptr) as mxml_type_t, byval sax as mxml_sax_cb_t, byval sax_data as any ptr) as mxml_node_t ptr

declare function mxmlSetCDATA(byval node as mxml_node_t ptr, byval data as const zstring ptr) as long
declare function mxmlSetCustom(byval node as mxml_node_t ptr, byval data as any ptr, byval destroy as mxml_custom_destroy_cb_t) as long
declare sub      mxmlSetCustomHandlers(byval load as mxml_custom_load_cb_t, byval save as mxml_custom_save_cb_t)
declare function mxmlSetElement(byval node as mxml_node_t ptr, byval name as const zstring ptr) as long
declare sub      mxmlSetErrorCallback(byval cb as mxml_error_cb_t)
declare function mxmlSetInteger(byval node as mxml_node_t ptr, byval integer as long) as long
declare function mxmlSetOpaque(byval node as mxml_node_t ptr, byval opaque as const zstring ptr) as long
declare function mxmlSetReal(byval node as mxml_node_t ptr, byval real as double) as long
declare function mxmlSetText(byval node as mxml_node_t ptr, byval whitespace as long, byval string as const zstring ptr) as long
declare function mxmlSetTextf(byval node as mxml_node_t ptr, byval whitespace as long, byval format as const zstring ptr, ...) as long
declare function mxmlSetUserData(byval node as mxml_node_t ptr, byval data as any ptr) as long
declare sub      mxmlSetWrapMargin(byval column as long)

declare function mxmlWalkNext(byval node as mxml_node_t ptr, byval top as mxml_node_t ptr, byval descend as long) as mxml_node_t ptr
declare function mxmlWalkPrev(byval node as mxml_node_t ptr, byval top as mxml_node_t ptr, byval descend as long) as mxml_node_t ptr

declare sub      mxml_error(byval format as const zstring ptr, ...)
declare function mxml_ignore_cb(byval node as mxml_node_t ptr)  as mxml_type_t
declare function mxml_integer_cb(byval node as mxml_node_t ptr) as mxml_type_t
declare function mxml_opaque_cb(byval node as mxml_node_t ptr)  as mxml_type_t
declare function mxml_real_cb(byval node as mxml_node_t ptr)    as mxml_type_t

end extern

#endif ' __MINIXML_BI__
